import tg
from tg.caching import cached_property
from formencode import FancyValidator

try:
    from tgext.datahelpers.validators import SQLAEntityConverter
except ImportError:
    pass

try:
    from tgext.datahelpers.validators import MingEntityConverter
except ImportError:
    pass


class ModelEntityConverter(FancyValidator):
    def __init__(self, klass, slugified=False):
        super(FancyValidator, self).__init__(not_empty=True)
        self.model_name = klass
        self.slugified = slugified

    @cached_property
    def _converter(self):
        from turbopress import model
        if tg.config.get('use_sqlalchemy', False):
            return SQLAEntityConverter(getattr(model, self.model_name),
                                       session=model.DBSession,
                                       slugified=self.slugified)
        elif tg.config.get('use_ming', False):
            return MingEntityConverter(getattr(model, self.model_name),
                                       slugified=self.slugified)
        else:
            raise ValueError('Turbopress should be used with sqlalchemy or ming')

    def _convert_to_python(self, value, state):
        return self._converter._to_python(value, state)

    def _convert_from_python(self, value, state):
        return self._converter._from_python(value, state)

    def _validate_python(self, value, state):
        return self._converter.validate_python(value, state)

    _to_python = _convert_to_python
    _from_python = _convert_from_python
    validate_python = _validate_python


try:
    from ming.odm.odmsession import ODMCursor as MingCursor
except ImportError:
    class MingCursor(object):
        pass

try:
    from tg.support.paginate import _MingQueryWrapper
    tg_supports_ming_pagination = True
except ImportError:
    tg_supports_ming_pagination = False
    class _MingQueryWrapper(object):
        def __init__(self, obj):
            self.obj = obj

        def __getitem__(self, range):
            return self.obj.skip(range.start).limit(range.stop-range.start)

        def __len__(self):
            return self.obj.count()


def make_paginable(query):
    if not tg_supports_ming_pagination and isinstance(query, MingCursor):
        return _MingQueryWrapper(query)
    return query


from tg import expose, request
from turbopress import model
from tgext.pluggable import plug_url, app_model
from turbopress.lib.forms import SearchForm

WEEK_EXPIRE = 3600*24*7


@expose('genshi:turbopress.templates.articles')
def articles(articles=None, blog=None):
    if articles is None:
        articles = model.Article.get_published(blog)
    return dict(articles=articles)


@expose('genshi:turbopress.templates.article_preview')
def article_preview(article):
    user = 'anonymous'
    if request.identity:
        try:
            user = request.identity['user']._id
        except AttributeError:
            from tgext.pluggable import primary_key
            user = getattr(request.identity['user'], primary_key(app_model.User).key)

    return dict(article=article, tg_cache=dict(key='%s-%s-%s' % (article.uid, user,
                                                                 article.update_date.strftime('%Y-%m-%d_%H:%M:%S')),
                                               expire=WEEK_EXPIRE,
                                               type='memory'))


@expose('genshi:turbopress.templates.tagcloud')
def tagcloud(blog, tags):
    return dict(blog=blog, tags=tags)


@expose('genshi:turbopress.templates.search')
def search(blog, value=''):
    return dict(form=SearchForm, value=dict(blog=blog, text=value),
                action=plug_url('turbopress', '/search'))


@expose('genshi:turbopress.templates.articles_excerpts')
def excerpts(articles=None, blog=None):
    if articles is None:
        articles = model.Article.get_published(blog)
    return dict(articles=articles)


@expose('genshi:turbopress.templates.excerpt')
def excerpt(article):
    return article_preview(article)



import os
from datetime import datetime
import shutil

import tg
from tg.caching import cached_property
from tg import request

from ming.odm import FieldProperty, ForeignIdProperty, RelationProperty, MapperExtension
from ming.odm.declarative import MappedClass
from ming import schema as s, DESCENDING

from tgext.datahelpers.utils import slugify
from tgext.datahelpers.attachments import AttachedFile
from turbopress.model import DBSession
from tgext.pluggable import app_model

import mimetypes

mimetypes.init()

AttachmentType = tg.config.get('_turbopress', {}).get('attachment_type', AttachedFile)
ENABLE_ARTICLES_CACHE = tg.config.get('_turbopress', {}).get('enable_cache', False)
ARTICLE_CACHE_EXPIRE = tg.config.get('_turbopress', {}).get('cache_expire', 30 * 60)


class Blog(MappedClass):
    class __mongometa__:
        name = 'turbopress_blogs'
        session = DBSession
        indexes = [('name',)]

    _id = FieldProperty(s.ObjectId)
    name = FieldProperty(s.String, required=True, if_missing=u"Untitled")

    articles = RelationProperty('Article')

    @cached_property
    def uid(self):
        # Provided for seamless compatibility with SQLAlchemy
        return str(self._id)


class ArticleExt(MapperExtension):
    def before_update(self, instance, state, sess):
        instance.update_date = datetime.utcnow()


class Article(MappedClass):
    class __mongometa__:
        name = 'turbopress_articles'
        session = DBSession
        indexes = [('title',), ('published',), ('blog_id',), ('publish_date',)]
        extensions = [ArticleExt]

    _id = FieldProperty(s.ObjectId)
    title = FieldProperty(s.String, if_missing=u"Untitled")
    published = FieldProperty(s.Bool, if_missing=False)
    private = FieldProperty(s.Int, if_missing=0)
    tags = FieldProperty([s.String], if_missing=[])

    author_id = ForeignIdProperty(app_model.User)
    author = RelationProperty('User')

    blog_id = ForeignIdProperty(Blog)
    blog = RelationProperty('Blog')

    attachments = RelationProperty('Attachment')

    publish_date = FieldProperty(s.DateTime, if_missing=datetime.utcnow)
    update_date = FieldProperty(s.DateTime, if_missing=datetime.utcnow)

    description = FieldProperty(s.String, if_missing=u'Empty article, edit or delete this')
    content = FieldProperty(s.String, if_missing=u'')

    @cached_property
    def uid(self):
        # Provided for seamless compatibility with SQLAlchemy
        return str(self._id)

    @cached_property
    def slug(self):
        return slugify(self, self.title)

    @staticmethod
    def get_tagcloud(blog=None):
        if blog:
            try:
                blog = blog._id
            except:
                blog = Blog.query.find({'name': blog}).first()
                if not blog:
                    return []
                blog = blog._id

        pipeline = (
            {'$project': {'tags': 1}},
            {'$unwind': "$tags"},
            {'$group': {'_id': '', 'tags': {'$addToSet': '$tags'}}}
        )

        if blog:
            pipeline = ({'$match': {'blog_id': blog}},) + pipeline

        values = DBSession.impl.db.turbopress_articles.aggregate(pipeline)
        try:
            values = values['result'][0]['tags']
        except:
            values = []
        return values

    @staticmethod
    def get_published(blog=None):
        now = datetime.utcnow()
        filter = {'published': True,
                  'publish_date': {'$lte': now}}

        if blog:
            if not hasattr(blog, '_id'):
                blog = Blog.query.find({'name': blog}).first()

            try:
                filter.update({'blog_id': blog._id})
            except AttributeError:
                pass

        articles = Article.query.find(filter).sort([('publish_date', DESCENDING)])
        return articles

    @staticmethod
    def get_all(blog=None):
        filter = {}

        if blog:
            if not hasattr(blog, '_id'):
                blog = Blog.query.find({'name': blog}).first()

            try:
                filter.update({'blog_id': blog._id})
            except AttributeError:
                pass

        articles = Article.query.find(filter).sort([('publish_date', DESCENDING)])
        return articles.all()

    @property
    def blog_name(self):
        blog = self.blog
        if blog:
            return blog.name
        else:
            return ''

    def is_owner(self, identity):
        if not identity:
            return False

        return identity['user'] == self.author

    @property
    def caching_options(self):
        if not ENABLE_ARTICLES_CACHE:
            return dict()

        userid = request.identity['user'].user_id if request.identity else 'None'
        try:
            return dict(key='%s-%s-%s-%s' % (userid, 'turbopress-article',
                                             self._id, self.update_date.strftime('%Y-%m-%d-%H-%M-%S')),
                        expire=ARTICLE_CACHE_EXPIRE, type="memory")
        except (IndexError, ValueError):
            return dict()


class AttachmentExt(MapperExtension):
    def before_delete(self, instance, state, sess):
        try:
            shutil.rmtree(os.path.dirname(instance.content.local_path))
        except:
            pass


class Attachment(MappedClass):
    class __mongometa__:
        name = 'turbopress_attachments'
        session = DBSession
        indexes = [('article_id',)]
        extensions = [AttachmentExt]

    _id = FieldProperty(s.ObjectId)
    name = FieldProperty(s.String, required=True)

    article_id = ForeignIdProperty(Article)
    article = RelationProperty(Article)

    class AttachedFileProperty(FieldProperty):
        def __init__(self):
            super(self.__class__, self).__init__({'file': s.String(required=True),
                                                  'uuid': s.String(required=True),
                                                  'filename': s.String(required=True)})

        def __get__(self, instance, owner=None):
            if instance is None: return self
            v = super(self.__class__, self).__get__(instance)
            return AttachmentType(**v)

        def __set__(self, instance, value):
            value.write()

            value = {'uuid': value.uuid,
                     'file': value.local_path,
                     'filename': value.filename}
            return FieldProperty.__set__(self, instance, value)

    content = AttachedFileProperty()

    @cached_property
    def uid(self):
        # Provided for seamless compatibility with SQLAlchemy
        return str(self._id)

    @property
    def mimetype(self):
        return mimetypes.guess_type(self.content.local_path, False)[0]

    @property
    def url(self):
        return self.content.url



from datetime import datetime
from .base import configure_app, create_app, flush_db_changes
from turbopress import model


class TurboPressAuthControllerTests(object):
    BLOG = ''

    @classmethod
    def blog_url(cls, url):
        if cls.BLOG:
            return url + '/' + cls.BLOG
        return url

    def setup(self):
        self.app = create_app(self.app_config, True)
        if self.BLOG:
            # Create another fake blog to make sure you don't get the wrong one.
            model.provider.create(model.Blog, {'name': 'ANOTHER BLOG'})
            model.provider.create(model.Blog, {'name': 'ANOTHER BLOG2'})
            model.provider.create(model.Blog, {'name': 'ANOTHER BLOG2'})
            model.provider.create(model.Blog, {'name': self.BLOG})
            flush_db_changes()

    def test_blog_new_authorized(self):
        resp = self.app.get(self.blog_url('/press/new'), status=302)
        assert 'http://localhost/press/edit' in resp.location

        resp = resp.follow(extra_environ={'REMOTE_USER': 'user'})
        assert 'action="/press/save"' in resp.text

    def test_blog_save(self):
        resp = self.app.get(self.blog_url('/press/new'), status=302)

        __, posts = model.provider.query(model.Article)
        post = posts[0]

        now = datetime.utcnow().strftime('%Y-%m-%dT%H:%M')
        resp = self.app.post('/press/save', {'uid': post.uid,
                                                  'title': '+Title',
                                                  'content': '+Content',
                                                  'description': '+Description',
                                                  'publish_date': now},
                                  status=302)

        __, posts = model.provider.query(model.Article)
        post = posts[0]
        assert post.title == '+Title'
        assert post.content == '+Content'
        assert post.description == '+Description'

        # Make sure that new posts are not published
        resp = self.app.get(self.blog_url('/press/index'))
        assert '''<div id="turbopress_articles">
</div>''' in resp.text, resp.text

    def test_blog_prevent_publish_empty(self):
        resp = self.app.get(self.blog_url('/press/new'), status=302)
        __, posts = model.provider.query(model.Article)
        post = posts[0]
        post_uid = post.uid

        resp = self.app.get('/press/publish', {'article': post_uid}, status=302)
        resp = resp.follow()

        assert 'Cannot publish an empty article' in resp.text

    def test_blog_publish(self):
        resp = self.app.get(self.blog_url('/press/new'), status=302)
        __, posts = model.provider.query(model.Article)
        post = posts[0]
        post_uid = post.uid

        post.content = '+Content'
        flush_db_changes()

        self.app.get('/press/publish', {'article': post_uid}, status=302)

        resp = self.app.get(self.blog_url('/press/index'))
        assert ('<a href="/press/view/%s">Untitled</a>' % post_uid) in resp.text, resp.text

    def test_blog_hide(self):
        resp = self.app.get(self.blog_url('/press/new'), status=302)
        __, posts = model.provider.query(model.Article)
        post = posts[0]
        post_uid = post.uid

        post.content = '+Content'
        flush_db_changes()

        self.app.get('/press/publish', {'article': post_uid}, status=302)
        self.app.get('/press/hide', {'article': post_uid}, status=302)

        # Make sure that new posts are not published
        resp = self.app.get(self.blog_url('/press/index'))
        assert '''<div id="turbopress_articles">
</div>''' in resp.text, resp.text


class TestTurboPressControllerAuthenticatedSQLA(TurboPressAuthControllerTests):
    @classmethod
    def setupClass(cls):
        cls.app_config = configure_app('sqlalchemy')


class TestTurboPressControllerAuthenticatedMing(TurboPressAuthControllerTests):
    @classmethod
    def setupClass(cls):
        cls.app_config = configure_app('ming')




import os
from datetime import datetime

import tg
from tg import request, config

from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.ext.hybrid import hybrid_property, Comparator
from sqlalchemy import Table, ForeignKey, Column, Index
from sqlalchemy.types import Unicode, Integer, DateTime
from sqlalchemy.orm import backref, relation
from sqlalchemy import event, func, select

from tgext.datahelpers.utils import slugify
from tgext.datahelpers.fields import Attachment as DatahelpersAttachment, AttachedFile
from turbopress.model import DBSession
from tgext.pluggable import app_model, primary_key

import mimetypes
mimetypes.init()

DeclarativeBase = declarative_base()

AttachmentType = tg.config.get('_turbopress', {}).get('attachment_type', AttachedFile)
ENABLE_ARTICLES_CACHE = tg.config.get('_turbopress', {}).get('enable_cache', False)
ARTICLE_CACHE_EXPIRE = tg.config.get('_turbopress', {}).get('cache_expire', 30*60)


class Blog(DeclarativeBase):
    __tablename__ = 'turbopress_blogs'

    uid = Column(Integer, autoincrement=True, primary_key=True)
    name = Column(Unicode(100), nullable=False, default=u"Untitled", index=True)


class Article(DeclarativeBase):
    __tablename__ = 'turbopress_articles'

    uid = Column(Integer, autoincrement=True, primary_key=True)
    title = Column(Unicode(150), nullable=False, default=u"Untitled", index=True)
    published = Column(Integer, nullable=False, default=0, index=True)
    private = Column(Integer, nullable=False, default=0)

    author_id = Column(Integer, ForeignKey(primary_key(app_model.User)))
    author = relation(app_model.User, backref=backref('articles'))

    blog_id = Column(Integer, ForeignKey(primary_key(Blog)), nullable=True, index=True)
    blog = relation(Blog, backref=backref('articles'), lazy='joined')

    publish_date = Column(DateTime, nullable=False, default=datetime.utcnow, index=True)
    update_date = Column(DateTime, nullable=False, default=datetime.utcnow)
    description = Column(Unicode(1024), nullable=False,
                         default=u'Empty article, edit or delete this')
    content = Column(Unicode(64000), nullable=False, default=u'')

    @hybrid_property
    def tags(self):
        return [x[0] for x in DBSession.query(Tagging.name).filter(Tagging.article_id == self.uid)]

    @property
    def slug(self):
        return slugify(self, self.title)

    @tags.setter
    def tags(self, value):
        self.tagging = [Tagging(blog_id=self.blog_id, article_id=self.uid, name=t) for t in value]

    @tags.comparator
    def tags(cls):
        class TaggingComparator(Comparator):
            def __eq__(self, other):
                return self.expression.any(Tagging.name == other)
        return TaggingComparator(cls.tagging)

    @staticmethod
    def get_tagcloud(blog=None):
        tags = DBSession.query(Tagging.name)

        if blog:
            try:
                blog = blog.uid
            except:
                blog = DBSession.query(Blog).filter_by(name=blog).first()
                if not blog:
                    return []

                blog = blog.uid

            tags = tags.filter_by(blog_id=blog)

        return tags.group_by(Tagging.name).all()

    @staticmethod
    def get_published(blog=None):
        now = datetime.utcnow()
        articles = DBSession.query(Article).filter_by(published=True)\
                                           .filter(Article.publish_date<=now)

        if blog:
            try:
                blog = blog.name
            except:
                pass
            articles = articles.join(Blog).filter(Blog.name==blog)

        articles = articles.order_by(Article.publish_date.desc())
        return articles

    @staticmethod
    def get_all(blog=None):
        articles = DBSession.query(Article)
        if blog:
            try:
                blog = blog.name
            except:
                pass
            articles = articles.join(Blog).filter(Blog.name==blog)
        articles = articles.order_by(Article.publish_date.desc())
        return articles

    @property
    def blog_name(self):
        blog = self.blog
        if blog:
            return blog.name
        else:
            return ''

    def is_owner(self, identity):
        if not identity:
            return False

        return identity['user'] == self.author

    @property
    def caching_options(self):
        if not ENABLE_ARTICLES_CACHE:
            return dict()

        userid = request.identity['user'].user_id if request.identity else 'None'
        try:
            return dict(key='%s-%s-%s-%s' % (userid, 'turbopress-article',
                                             self.uid,
                                             self.update_date.strftime('%Y-%m-%d-%H-%M-%S')),
                                             expire=ARTICLE_CACHE_EXPIRE, type="memory")
        except (IndexError, ValueError):
            return dict()


class Attachment(DeclarativeBase):
    __tablename__ = 'turbopress_attachments'

    uid = Column(Integer, autoincrement=True, primary_key=True)
    name = Column(Unicode(16), nullable=False)
    content = Column(DatahelpersAttachment(AttachmentType))

    article_id = Column(Integer, ForeignKey(Article.uid))
    article = relation(Article, backref=backref('attachments', cascade='all, delete-orphan'))

    @staticmethod
    def delete_file(mapper, connection, obj):
        try:
            os.unlink(obj.content.local_path)
        except:
            pass

    @property
    def mimetype(self):
        return mimetypes.guess_type(self.content.local_path, False)[0]

    @property
    def url(self):
        return self.content.url

event.listen(Attachment, 'before_delete', Attachment.delete_file)


class Tagging(DeclarativeBase):
    __tablename__ = 'turbopress_tagging'

    blog_id = Column(Integer, ForeignKey(Blog.uid), index=True)
    article_id = Column(Integer, ForeignKey(Article.uid), nullable=False, primary_key=True)
    name = Column(Unicode(64), nullable=False, primary_key=True)

    article = relation(Article, backref=backref('tagging', cascade='all, delete-orphan'))

    @staticmethod
    def on_change(mapper, connection, obj):
        obj.name = obj.name.strip().lower()

event.listen(Tagging, 'before_insert', Tagging.on_change)
event.listen(Tagging, 'before_update', Tagging.on_change)



# -*- coding: utf-8 -*-
"""Main Controller"""
import tg
from tg import TGController
from tg import expose, flash, require, url, request, redirect, validate, config, abort
from tg.decorators import cached_property, paginate
from tg.i18n import ugettext as _, lazy_ugettext as l_
from tg import predicates

from turbopress import model
from turbopress.helpers import *
from turbopress.lib.forms import UploadForm, get_article_form, DataGrid, \
    inject_datagrid_resources, SpinnerIcon
from tgext.datahelpers.attachments import AttachedFile
from tgext.pluggable import plug_url, plug_redirect
from webhelpers.html.builder import HTML
from datetime import datetime
from tgext.ajaxforms.ajaxform import formexpose
from tgext.crud import EasyCrudRestController
from turbopress.lib.dal import ModelEntityConverter, make_paginable

AttachmentType = config.get('_turbopress',{}).get('attachment_type', AttachedFile)

upload_form = UploadForm()
articles_table = DataGrid(fields=[(l_('Actions'), lambda row:link_buttons(row, 'edit', 'delete', 'hide', 'publish')),
                                  (l_('Blog'), lambda row:row.blog and row.blog.name),
                                  (l_('Title'), lambda row:HTML.a(row.title,
                                                                  href=plug_url('turbopress', '/view/%s'%row.uid,
                                                                                lazy=True))),
                                  (l_('Tags'), comma_separated_tags),
                                  (l_('Author'), 'author'),
                                  (l_('Publishing'), lambda row:format_published(row) + ', ' + format_date(row))])

attachments_table = DataGrid(fields=[(l_('Actions'), lambda row:link_buttons(row, 'rmattachment')),
                                     (l_('Name'), 'name'),
                                     (l_('Preview'), attachment_preview),
                                     (l_('Url'), lambda row:format_link(row.url))])


class BlogsController(EasyCrudRestController):
    allow_only = predicates.has_permission('turbopress')
    title = "Manage Blogs"

    __form_options__ = {
        '__omit_fields__': ['articles', 'uid', '_id']
    }

    __table_options__ = {
        '__omit_fields__': ['uid', '_id', 'articles']
    }


class RootController(TGController):
    @cached_property
    def blogs(self):
        class ModelBoundBlogsController(BlogsController):
            model = model.Blog

        return ModelBoundBlogsController(model.DBSession.wrapped_session)

    @expose('genshi:turbopress.templates.index')
    @paginate('articles')
    def index(self, blog='', *args, **kw):
        articles = make_paginable(model.Article.get_published(blog))
        tags = model.Article.get_tagcloud(blog)
        return dict(articles=articles, tags=tags, blog=blog, search='')

    _default = index

    @expose('genshi:turbopress.templates.article')
    @validate(dict(article=ModelEntityConverter('Article', slugified=True)), error_handler=index)
    def view(self, article, **kw):
        visible = False

        if article.published and article.publish_date <= datetime.utcnow():
            visible = True
        elif request.identity and article.author == request.identity['user']:
            visible = True
        elif request.identity and 'turbopress' in request.identity['groups']:
            visible = True

        if not visible:
            return plug_redirect('turbopress', '/')

        tg.hooks.notify('turbopress.before_view_article', args=(article, kw))

        return dict(article=article,
                    tg_cache=article.caching_options)

    @require(predicates.has_permission('turbopress'))
    @expose('genshi:turbopress.templates.manage')
    @paginate('articles')
    def manage(self, blog='', *args, **kw):
        articles = model.Article.get_all(blog)
        return dict(table=articles_table, articles=articles,
                    create_action=plug_url('turbopress', '/new/'+blog))

    @require(predicates.has_permission('turbopress'))
    @expose('genshi:turbopress.templates.edit')
    def new(self, blog=None, **kw):
        inject_datagrid_resources(attachments_table)

        if 'uid' not in kw:
            if blog:
                __, blog = model.provider.query(model.Blog, filters=dict(name=blog))
                if blog:
                    blog = blog[0]

            article_data = {'author': request.identity['user'],
                            'blog': blog and blog}

            tg.hooks.notify('turbopress.before_create_article', args=(article_data, kw))
            article = model.provider.create(model.Article, article_data)
            tg.hooks.notify('turbopress.after_create_article', args=(article, kw))
        else:
            article = model.provider.get_obj(model.Article, params=dict(uid=kw['uid'],
                                                                        _id=kw['uid']))

        return plug_redirect('turbopress', '/edit/%s' % article.uid)

    @require(predicates.has_permission('turbopress'))
    @expose('genshi:turbopress.templates.edit')
    def edit(self, uid=None, *args, **kw):
        if not uid:
            abort(404)

        article = model.provider.get_obj(model.Article, params=dict(uid=uid,
                                                                    _id=uid))
        if not article:
            abort(404)

        inject_datagrid_resources(attachments_table)

        value = {
            'uid': article.uid,
            'title': article.title,
            'description': article.description,
            'tags': comma_separated_tags(article),
            'publish_date': article.publish_date.strftime('%Y-%m-%dT%H:%M'),
            'content': article.content
        }

        tg.hooks.notify('turbopress.before_edit_article', args=(article, value))

        return dict(article=article, value=value, blog=article.blog and article.blog.name or '',
                    form=get_article_form(), action=plug_url('turbopress', '/save'),
                    upload_form=upload_form, upload_action=plug_url('turbopress', '/attach'))

    @require(predicates.has_permission('turbopress'))
    @validate(get_article_form(), error_handler=edit)
    @expose()
    def save(self, *args, **kw):
        article = model.provider.get_obj(model.Article, params=dict(uid=kw['uid'],
                                                                    _id=kw['uid']))

        tg.hooks.notify('turbopress.before_save_article', args=(article, kw))

        article.title = kw['title']
        article.description = kw['description']
        article.content = kw['content']
        article.publish_date = kw['publish_date']
        article.update_date = datetime.utcnow()
        article.tags = tuple(set(x.strip().lower() for x in kw['tags'].split(',')))

        flash(_('Articles successfully saved'))
        return plug_redirect('turbopress', '/manage/'+article.blog_name)

    @require(predicates.has_permission('turbopress'))
    @formexpose(upload_form, 'turbopress.templates.attachments')
    def upload_form_show(self, **kw):
        article = model.provider.get_obj(model.Article, params=dict(uid=kw['article'],
                                                                    _id=kw['article']))

        return dict(value=kw, table=attachments_table, attachments=article.attachments)

    @require(predicates.has_permission('turbopress'))
    @validate(upload_form, error_handler=upload_form_show)
    @expose('genshi:turbopress.templates.attachments')
    def attach(self, **kw):
        article = model.provider.get_obj(model.Article, params=dict(uid=kw['article'],
                                                                    _id=kw['article']))

        attachment_data = dict(name=kw['name'],
                               article=article,
                               content=AttachmentType(kw['file'].file,
                                                      kw['file'].filename))

        tg.hooks.notify('turbopress.on_attachment_upload', args=(article, attachment_data))

        model.provider.create(model.Attachment, attachment_data)

        return dict(value=dict(article=article.uid),
                    ajaxform=upload_form,
                    ajaxform_id='attachments_form',
                    ajaxform_action=upload_form.action,
                    ajaxform_spinner=SpinnerIcon(),
                    table=attachments_table,
                    attachments=article.attachments)

    @require(predicates.has_permission('turbopress'))
    @validate(dict(attachment=ModelEntityConverter('Attachment')))
    @expose()
    def rmattachment(self, attachment):
        article = attachment.article

        tg.hooks.notify('turbopress.on_attachment_remove', args=(article, attachment))

        model.provider.delete(model.Attachment, params=dict(uid=attachment.uid,
                                                            _id=attachment.uid))

        flash(_('Attachment successfully removed'))
        return plug_redirect('turbopress', '/edit/%s'%article.uid)

    @require(predicates.has_permission('turbopress'))
    @validate(dict(article=ModelEntityConverter('Article')), error_handler=manage)
    @expose()
    def publish(self, article):
        if not article.content:
            flash(_('Cannot publish an empty article'))
        else:
            article.published=True
            flash(_('Article published'))

        tg.hooks.notify('turbopress.on_article_publishing', args=(article, True))
        return plug_redirect('turbopress', '/manage/'+article.blog_name)

    @require(predicates.has_permission('turbopress'))
    @validate(dict(article=ModelEntityConverter('Article')), error_handler=manage)
    @expose()
    def hide(self, article):
        article.published=False
        flash(_('Article hidden'))

        tg.hooks.notify('turbopress.on_article_publishing', args=(article, False))
        return plug_redirect('turbopress', '/manage/'+article.blog_name)

    @require(predicates.has_permission('turbopress'))
    @validate(dict(article=ModelEntityConverter('Article')), error_handler=manage)
    @expose()
    def delete(self, article):
        tg.hooks.notify('turbopress.on_article_delete', args=(article))

        model.provider.delete(model.Article, params=dict(uid=article.uid,
                                                         _id=article.uid))

        flash(_('Article successfully removed'))
        return plug_redirect('turbopress', '/manage/'+article.blog_name)

    @expose('genshi:turbopress.templates.index')
    @paginate('articles')
    def search(self, blog='', text=None, tags=None):
        filters = dict()
        extra_args = {}
        if text:
            filters['content'] = text
            extra_args['substring_filters'] = ['content']

        if blog:
            __, blogs = model.provider.query(model.Blog, filters=dict(name=blog))
            blog_entry = blogs[0]

            # While sprox SA provider does cast values, the Ming one
            # doesn't. So we must ensure that we actually use an ObjectId
            # and not a string, or it won't find anything.
            filters['blog_id'] = getattr(blog_entry, '_id', blog_entry.uid)

        if tags:
            filters['tags'] = tags

        if 'tags' not in filters and 'content' not in filters:
            flash(_('No search rule has been specified'))
            return plug_redirect('turbopress', '/'+blog)

        count, articles = model.provider.query(model.Article,
                                               filters=filters,
                                               **extra_args)

        blog_tags = model.Article.get_tagcloud(blog)
        return dict(articles=articles, tags=blog_tags, blog=blog, search=text)



import tg
from tg.i18n import lazy_ugettext as l_
from tgext.ajaxforms import ajaxloaded
from tgext.pluggable import plug_url
from turbopress.lib.validators import NaiveDateTimeValidator
from tw2.core import CSSLink, Required
from tw2.forms import DataGrid
from tw2.forms import ListForm, TextField, TextArea, HiddenField, FileField, SubmitButton
from formencode.validators import UnicodeString, FieldStorageUploadConverter


class SpinnerIcon(object):
    @property
    def link(self):
        return tg.url('/_pluggable/turbopress/images/spinner.gif')


def inject_datagrid_resources(dg):
    resources = [r.req() for r in dg.resources]
    for r in resources:
        r.prepare()


class ArticleForm(ListForm):
    uid = HiddenField()
    title = TextField(label='Title',
                      container_attrs={'class': 'form-group'}, css_class='form-control',
                      validator=UnicodeString(not_empty=True, outputEncoding=None))
    description = TextField(label='Description',
                            container_attrs={'class': 'form-group'}, css_class='form-control',
                            validator=UnicodeString(outputEncoding=None),
                            placeholder=l_('If empty will be extracted from the content'))
    tags = TextField(label='Tags',
                     container_attrs={'class': 'form-group'}, css_class='form-control',
                     validator=UnicodeString(outputEncoding=None),
                     placeholder=l_('tags, comma separated'))
    content = TextArea(label=None, key='content', name='content', id="article_content",
                       container_attrs={'class': 'form-group'}, css_class='form-control',
                       validator=UnicodeString(not_empty=True, outputEncoding=None))
    publish_date = TextField(label='Publish Date', type='datetime-local',
                             container_attrs={'class': 'form-group'}, css_class='form-control',
                             validator=NaiveDateTimeValidator(not_empty=True))

    submit = SubmitButton(value=l_('Save'), css_class="btn btn-primary")

@ajaxloaded
class UploadForm(ListForm):
    css_class = "form-inline"

    article = HiddenField()
    name = TextField(label=None, placeholder=l_('Name'),
                     container_attrs={'class': 'form-group'}, css_class='form-control',
                     validator=UnicodeString(not_empty=True, outputEncoding=None))
    file = FileField(label=None,
                     container_attrs={'class': 'form-group'}, css_class='form-control',
                     validator=FieldStorageUploadConverter(not_empty=True))

    action = plug_url('turbopress', '/attach', lazy=True)
    ajaxurl = plug_url('turbopress', '/upload_form_show', lazy=True)

    submit = SubmitButton(value='Attach', label=None, css_class="btn btn-info")


class SearchForm(ListForm):
    css_class = "form-inline"

    blog = HiddenField()
    text = TextField(label=None, validator=UnicodeString(not_empty=True, outputEncoding=None),
                     container_attrs={'class': 'form-group'}, css_class='form-control',
                     placeholder=l_('Search...'))
    submit_btn = SubmitButton(value='Search', container_attrs={'class': 'form-group'},
                              css_class="form-control btn btn-default")
    submit = None


def get_article_form():
    config = tg.config['_turbopress']

    article_form = config.get('form_instance')
    if not article_form:
        form_path = config.get('form', 'turbopress.lib.forms.ArticleForm')
        module, form_name = form_path.rsplit('.', 1)
        module = __import__(module, fromlist=form_name)
        form_class = getattr(module, form_name)
        article_form = config['form_instance'] = form_class()

    return article_form



from test_auth_controller import TurboPressAuthControllerTests
from .base import configure_app


class TestSubBlogAuthenticatedSQLA(TurboPressAuthControllerTests):
    BLOG = 'testblog'

    @classmethod
    def setupClass(cls):
        cls.app_config = configure_app('sqlalchemy')


class TestSubBlogAuthenticatedMing(TurboPressAuthControllerTests):
    BLOG = 'testblog'

    @classmethod
    def setupClass(cls):
        cls.app_config = configure_app('ming')




# -*- coding: utf-8 -*-
"""The tgapp-turbopress package"""
import tg
from tg.configuration import milestones


def plugme(app_config, options):
    tg.config['_turbopress'] = options

    from turbopress import model
    milestones.config_ready.register(model.configure_models)
    milestones.environment_loaded.register(model.configure_provider)
    return dict(appid='press', global_helpers=False)




# -*- coding: utf-8 -*-
"""Setup the Turbopress application"""

from turbopress import model
from tgext.pluggable import app_model


def bootstrap(command, conf, vars):
    print 'Bootstrapping Turbopress...'

    p = model.provider.create(app_model.Permission, dict(permission_name='turbopress',
                                                         description='Can manage blogs and posts'))

    __, g = model.provider.query(app_model.Group, filters=dict(group_name='managers'), limit=1)
    if g:
        g = g[0]
        g.permissions = g.permissions + [p]
        model.DBSession.flush()



from datetime import datetime
from formencode.validators import FancyValidator, Invalid


class NaiveDateTimeValidator(FancyValidator):
    def _to_python(self, value, status):
        date_string = value.strip()
        try:
            date = datetime.strptime(date_string, '%Y-%m-%dT%H:%M')
        except:
            raise Invalid('date must be in format yyyy-mm-ddThh:mm', value, status)
        return date

    _convert_to_python = _to_python


# -*- coding: utf-8 -*-
import sys, os

try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages

install_requires=[
    "TurboGears2 >= 2.3.0",
    "formencode",
    "webhelpers",
    "tw2.forms",
    "tw2.jquery",
    "tgext.pluggable",
    "tgext.datahelpers",
    "tgext.ajaxforms",
    "tgext.crud",
    "sprox >= 0.9.1",
    "genshi"
]

testpkgs=['WebTest >= 1.2.3',
          'nose',
          'coverage',
          'ming',
          'sqlalchemy',
          'zope.sqlalchemy',
          'repoze.who'
]

here = os.path.abspath(os.path.dirname(__file__))
try:
    README = open(os.path.join(here, 'README.rst')).read()
except IOError:
    README = ''

setup(
    name='tgapp-turbopress',
    version='0.0.1',
    description='Pluggable Blog for TurboGears2 supporting SQLAlchemy and MongoDB',
    long_description=README,
    author='AXANT',
    author_email='tech@axant.it',
    url='http://bitbucket.org/axant/tgapp-turbopress',
    keywords='turbogears2.application',
    packages=find_packages(exclude=['ez_setup']),
    install_requires=install_requires,
    test_suite='nose.collector',
    tests_require=testpkgs,
    extras_require={
       # Used by Drone.io
       'testing':testpkgs,
    },
    include_package_data=True,
    package_data={'tgapp.turbopress': ['i18n/*/LC_MESSAGES/*.mo',
                                 'templates/*/*',
                                 'public/*/*']},
    entry_points="""
    """,
    zip_safe=False
)



from datetime import datetime
from .base import configure_app, create_app, flush_db_changes
from turbopress import model


class TurboPressControllerTests(object):
    def setup(self):
        self.app = create_app(self.app_config, False)

    def test_index(self):
        resp = self.app.get('/')
        assert 'HELLO' in resp.text

    def test_blog_index(self):
        a = model.provider.create(model.Article, {'title': 'Hello',
                                                  'content': 'World',
                                                  'published': True})
        article_id = a.uid
        flush_db_changes()

        resp = self.app.get('/press')
        assert 'turbopress_articles' in resp.text
        assert ('<a href="/press/view/%s">Hello</a>' % article_id) in resp.text, resp.text

    def test_blog_new_unauth(self):
        self.app.get('/press/new', status=401)


class TestTurboPressControllerSQLA(TurboPressControllerTests):
    @classmethod
    def setupClass(cls):
        cls.app_config = configure_app('sqlalchemy')


class TestTurboPressControllerMing(TurboPressControllerTests):
    @classmethod
    def setupClass(cls):
        cls.app_config = configure_app('ming')

